/*
 * Pipe pseudo-filesystem
 * Copyright (c) 2004, David H. Hovemeyer <daveho@cs.umd.edu>
 * $Revision: 1.3 $
 *
 * This is free software.  You are permitted to use,
 * redistribute, and modify it as specified in the file "COPYING".
 */

#ifndef GEEKOS_PIPEFS_H
#define GEEKOS_PIPEFS_H

/* Forward declarations */
struct File;
struct Pipe;

/* List containing all created pipes */
DEFINE_LIST(Pipes_List, Pipe);

/* The amount of storage to allocate for a pipe. */
#define PIPE_BUF_SIZE 4096

/* Maximal pipe name length */
#define MAX_PIPE_NAME 32

/* Pipe Data type */
struct Pipe
{
	char *name; //This will be 0 for unnamed pipes and !=0 for named pipes...
	ulong_t refCount; //Reference counter to this pipe
	char buffer[PIPE_BUF_SIZE]; //Buffer this pipe uses to store the data
	ulong_t readPos, writePos; //Current read/write position in the buffer
	struct Mutex mutex; //For mutex on the pipe
	DEFINE_LINK(Pipes_List, Pipe);
};

IMPLEMENT_LIST(Pipes_List, Pipe);

void Init_PIPEFS();
int Create_Pipe(struct File **pRead, struct File **pWrite);
int Create_Named_Pipe(const char *name, struct File **pRead, struct File **pWrite);

#endif /* GEEKOS_PIPEFS_H */

