/*
 * Synchronization primitives
 * Copyright (c) 2001, David H. Hovemeyer <daveho@cs.umd.edu>
 * $Revision: 1.13 $
 * 
 * This is free software.  You are permitted to use,
 * redistribute, and modify it as specified in the file "COPYING".
 */

#ifndef GEEKOS_SYNCH_H
#define GEEKOS_SYNCH_H

#include <geekos/kthread.h>
#include <geekos/list.h>

/*
 * Semaphores implementation:
 */

/* forward declarations */
struct Semaphore;
struct SemaphoreRefNode;

/* list that contains */
DEFINE_LIST(Semaphores_List, Semaphore);
DEFINE_LIST(Semaphores_Ref_List, SemaphoreRefNode);

/*
 * ProcessSemaphore is a list of all semaphores a process refers to
 * will be managed with the UserContext
 */
struct SemaphoreRefNode
{
	struct Semaphore *semaphore;
	DEFINE_LINK(Semaphores_Ref_List, SemaphoreRefNode);
};

struct Semaphore
{
	int id; //unique id along all semaphores
#define MAX_SEM_NAME_LENGTH 25
	char name[MAX_SEM_NAME_LENGTH+1]; //name to create/access the semaphore
	int refCount; //number of processes that are referring to this semaphore
	
	int count; //number of V() calls before P() may block
	
	struct Thread_Queue queue; //processes waiting for this semaphore

	DEFINE_LINK(Semaphores_List, Semaphore);
};

IMPLEMENT_LIST(Semaphores_List, Semaphore);
IMPLEMENT_LIST(Semaphores_Ref_List, SemaphoreRefNode);

//global: initialize global semaphore settings
void Init_Semaphores();

int Create_Semaphore(char *name, int nameLength, int initialCount);

int Semaphore_Down(int semId);
int Semaphore_Up(int semId);

int Destroy_Semaphore(int semId);
void Free_Semaphores_From_List(struct Semaphores_Ref_List *list);

/*
 * mutex states
 */
enum { MUTEX_UNLOCKED, MUTEX_LOCKED };

struct Mutex {
    int state;
    struct Kernel_Thread* owner;
    struct Thread_Queue waitQueue;
};

#define MUTEX_INITIALIZER { MUTEX_UNLOCKED, 0, THREAD_QUEUE_INITIALIZER }

struct Condition {
    struct Thread_Queue waitQueue;
};

void Mutex_Init(struct Mutex* mutex);
void Mutex_Lock(struct Mutex* mutex);
void Mutex_Unlock(struct Mutex* mutex);

void Cond_Init(struct Condition* cond);
void Cond_Wait(struct Condition* cond, struct Mutex* mutex);
void Cond_Signal(struct Condition* cond);
void Cond_Broadcast(struct Condition* cond);

#define IS_HELD(mutex) \
    ((mutex)->state == MUTEX_LOCKED && (mutex)->owner == g_currentThread)

#endif  /* GEEKOS_SYNCH_H */
